<?php

namespace App\Controller\Admin;

use App\Controller\Admin\Controller;

class MemberController extends Controller
{
    /**
     * $config.
     *
     * @var array
     */
    protected $config;

    public function __construct()
    {
        $this->config = config('member');
    }

    /**
     * 首頁
     */
    public function index()
    {
        $page = \Request::query('page', 1);
        $where = [['del', 0]];

        // 搜尋
        if (!empty($search = $this->search_check(\Request::query()))) {
            $where = array_merge($where, $search);
        }

        $data = \DB::table($this->config['table'])
                   ->where($where)
                   ->orderBy('created_at', 'desc')
                   ->paginate($this->config['page_show'], ['*'], 'page', $page)
                   ->withPath(base_path(\Request::path()))
                   ->appends(\Request::query());

        $view = \View::make("admin.{$this->config['folder']}.index", compact('data'))->with('config', $this->config)->render();

        return \Response::create($view)->send();
    }

    /**
     * 編輯頁面
     */
    public function edit($id)
    {
        $action = empty($id) ? 'add' : 'upd';

        $data = \DB::table($this->config['table'])->find($id);

        $view = \View::make("admin.{$this->config['folder']}.edit", compact('data', 'action'))->with('config', $this->config)->render();

        return \Response::create($view)->send();
    }

    /**
     * 新增
     */
    public function add($id)
    {
        // 需求狀況處理 - 需要返回 並顯示錯誤訊息
        $error = $this->condition_process();
        if (!empty($error)) {
            return back()->withErrors($error)->send();
        }

        // 針對 Request資料 做各種判斷處理
        $this->request_if();

        // 寫入資料表的內容
        $values = \Request::except('_token', '_method', 'action', 'confirm_pw');

        if (\DB::table($this->config['table'])->insert($values)) {
            return back()->with('message', '新增成功')->send();
        } else {
            return back()->withErrors('新增失敗')->send();
        }
    }

    /**
     * 更新
     */
    public function upd($id)
    {
        // 需求狀況處理 - 需要返回 並顯示錯誤訊息
        $error = $this->condition_process($id);
        if (!empty($error)) {
            return back()->withErrors($error)->send();
        }

        // 針對 Request資料 做各種判斷處理
        $this->request_if($id);

        // 寫入資料表的內容
        $values = \Request::except('_token', '_method', 'action', 'confirm_pw');

        if (\DB::table($this->config['table'])->where('id', $id)->update($values)) {
            return back()->with('message', '更新成功')->send();
        } else {
            return back()->withErrors('更新失敗')->send();
        }
    }

    /**
     * 單筆刪除 / 多筆刪除
     */
    public function del($id)
    {
        if (\Request::has('id') && is_array(\Request::input('id'))) {
            $id = \Request::input('id');
        }

        if ($this->soft_delete($id, $this->config['table'])) {
            return back()->with('message', '刪除成功')->send();
        } else {
            return back()->withErrors('刪除失敗')->send();
        }
    }

    /**
     * 需求狀況處理 - 需回傳錯誤訊息的各種狀況
     *
     * @param  int $id
     * @return string 狀況內容失敗時 回傳訊息
     */
    private function condition_process($id = 0)
    {
        //
    }

    /**
     * 透過 Request資料判斷 Request物件內容修改和執行狀況
     *
     * @param  int $id
     * @return void
     */
    private function request_if($id = 0)
    {
        // 新增時
        if ($id == 0) {
            \Request::merge(['block' => 0]);

            // 密碼加密
            if (\Request::has('password')) {
                \Request::merge(['password' => \Hash::make(\Request::input('password'))]);
            }
        }
    }

    /**
     * 更新密碼頁面
     */
    public function show_upd_pw($id)
    {
        $view = \View::make("admin.{$this->config['folder']}.upd_pw")->with('config', $this->config)->render();

        return \Response::create($view)->send();
    }

    /**
     * 更新密碼
     */
    public function upd_pw($id)
    {
        if (
            \DB::table($this->config['table'])->where('id', $id)->update(['password' => \Hash::make(\Request::input('password'))])
        ) {
            return back()->with('message', '密碼修改成功')->send();
        } else {
            return back()->withErrors('密碼修改失敗')->send();
        }
    }

    /**
     * Excel 行動選擇器
     */
    public function excel($action)
    {
        switch ($action) {
            case 'download_all':
                $this->excel_download_all();
                break;
            case 'import_add':
                $this->excel_import_add();
                break;
            default:
                return back()->withErrors('無對應的方法,動作參數：' . $action)->send();
                break;
        }
    }

    /**
     * Excel 多筆 導出 (橫式)
     */
    private function excel_download_all()
    {
        return with(new \App\Exports\MemberExport)->download()->send();
    }

    /**
     * Excel 多筆 導入新增
     */
    private function excel_import_add()
    {
        $rule = [
            'upload' => 'required|mimes:xls,xlsx',
        ];
        $rule_message = [
            'upload.required' => '未選擇任何檔案',
            'upload.mimes'    => '檔案格式有誤 限制：:values',
        ];
        // 驗證器
        validate($rule, $rule_message);

        $filePath = \Request::file('upload')->getRealPath();

        \Excel::import(new \App\Imports\MemberImport, \Request::file('upload'));
    }

    /**
     * Ajax 更新 選擇器
     */
    public function ajax_upd($id, $column)
    {
        switch ($column) {
            case 'block':
                $this->ajax_upd_block($id);
                break;
            case 'confirm':
                $this->ajax_upd_confirm($id);
                break;    
            default:
                if (
                    \DB::table($this->config['table'])->where('id', $id)->update([$column => \Request::input('value')])
                ) {
                    $message['success'] = '更新成功';
                } else {
                    $message['error'] = '更新失敗,欄位參數：' . $column;
                }

                return \JsonResponse::create($message)->send();
                break;
        }
    }

    /**
     * 黑名單
     */
    protected function ajax_upd_block($id)
    {
        $message = ['success' => '', 'error' => ''];

        if (
            \DB::table($this->config['table'])->where('id', $id)->update(['block' => \DB::raw("CASE block WHEN 1 THEN 0 WHEN 0 THEN 1 END")])
        ) {
            $message['success'] = '更新成功';
        } else {
            $message['error'] = '更新失敗';
        }

        return \JsonResponse::create($message)->send();
    }

    /**
     * 影片專區
     */
    protected function ajax_upd_confirm($id)
    {
        $message = ['success' => '', 'error' => ''];

        if (
            \DB::table($this->config['table'])->where('id', $id)->update(['confirm' => \DB::raw("CASE confirm WHEN 1 THEN 0 WHEN 0 THEN 1 END")])
        ) {
            $message['success'] = '更新成功';
        } else {
            $message['error'] = '更新失敗';
        }

        return \JsonResponse::create($message)->send();
    }
}