<?php

namespace App\Controller\Member;

// use App\Service\LINEloginService;

class RegisterController
{
    /**
     * $config.
     *
     * @var array
     */
    protected $config;

    public function __construct()
    {
        $this->config = config('member');
    }

    /**
     * 首頁
     */
    public function index()
    {
        $view = \View::make("web.{$this->config['folder']}.register")->render();

        return \Response::create($view)->send();
    }

    /**
     * 註冊
     */
    public function register()
    {
        if (\Session::get('validate.path.member-register') != 'pass') {
            return back()->withErrors('請重新操作 (Please re-operate)')->send();
        }

        $total = \DB::table($this->config['table'])->where([['account', \Request::input('account')], ['del', 0]])->count();

        if ($total > 0) {
            return back()->withErrors(lang('member.register.check.account-used'))->send();
        }

        \Request::merge(['password' => \Hash::make(\Request::input('password'))]);
        \Request::merge(['block' => 0]);

        $id = \DB::table($this->config['table'])->insertGetId(\Request::except('confirm_pw', '_token', 'action'));

        if (empty($id)) {
            return back()->withErrors(lang('member.register.check.add-fail'))->send();
        }

        \Session::put('m_check', \Crypt::encrypt($id));
        \Session::put('mid', $id);

        $url = \Session::has('return_path') ? \Session::get('return_path') : "{$this->config['link_tag']}/index";

        return redirect($url)->with('message', lang('member.register.check.add-success'))->send();
    }

    /**
     * FB 註冊
     */
    public function register_fb()
    {
        // 取得 FB資料
        $user = \Socialite::driver('facebook')->user();
        $user = \Collection::make($user)->toArray();

        if (empty($user) || empty($user['email']) || $user['email'] == '') {
            return redirect('member/login')->withErrors('取得FB資料失敗，請直接於本站進行會員操作.')->send();
        }

        $member = \DB::table($this->config['table'])->where([['account', $user['email']], ['del', 0]])->first();

        if (empty($member)) {
            // 要寫入的資料
            $var = [
				'account'  => $user['email'],
				'password' => \Hash::make($user['email']),
				'name'     => $user['name'],
				'block'    => 0,
            ];

            $id = \DB::table($this->config['table'])->insertGetId($var);

            if (empty($id)) {
                return back()->withErrors(lang('member.register.check.add-fail'))->send();
            }

            \Session::put('m_check', \Crypt::encrypt($id));
            \Session::put('mid', $id);

            return redirect("{$this->config['link_tag']}/password")->with('message', '加入會員成功, 預設密碼與您的信箱相同,請更改您的密碼.')->send();
        } else {
        	\Session::put('m_check', \Crypt::encrypt($member['id']));
            \Session::put('mid', $member['id']);

            $url = \Session::has('return_path') ? \Session::get('return_path') : "{$this->config['link_tag']}/index";

            return redirect($url)->with('message', lang('member.register.check.login'))->send();
        }
    }

    /**
     * google 註冊
     */
    public function register_google()
    {
        // 取得 google資料
        $user = \Socialite::driver('google')->user();
        $user = \Collection::make($user)->toArray();

        if (empty($user) || empty($user['email']) || $user['email'] == '') {
            return redirect('member/login')->withErrors('取得google資料失敗，請直接於本站進行會員操作.')->send();
        }

        $member = \DB::table($this->config['table'])->where([['account', $user['email']], ['del', 0]])->first();

        if (empty($member)) {
            // 要寫入的資料
            $var = [
				'account'  => $user['email'],
				'password' => \Hash::make($user['email']),
				'name'     => $user['name'],
				'block'    => 0,
            ];

            $id = \DB::table($this->config['table'])->insertGetId($var);

            if (empty($id)) {
                return back()->withErrors(lang('member.register.check.add-fail'))->send();
            }

            \Session::put('m_check', \Crypt::encrypt($id));
            \Session::put('mid', $id);

            return redirect("{$this->config['link_tag']}/password")->with('message', '加入會員成功, 預設密碼與您的信箱相同,請更改您的密碼.')->send();
        } else {
        	\Session::put('m_check', \Crypt::encrypt($member['id']));
            \Session::put('mid', $member['id']);

            $url = \Session::has('return_path') ? \Session::get('return_path') : "{$this->config['link_tag']}/index";

            return redirect($url)->with('message', lang('member.register.check.login'))->send();
        }
    }

    /**
     * LINE 註冊
     */
    // public function register_line()
    // {
    //     // LINE Receiving
    //     $authorization_code = \Request::input('code');
    //     $state = \Request::input('state');

    //     if ($state != csrf_token()) {
    //         return redirect('member/login')->withErrors('連線逾時，請重新操作一次 (If the connection is overdue, please re-operate once.)')->send();
    //     }

    //     // 取得 LINE 資料
    //     $user = with(new LINEloginService)->user($authorization_code);
    //     $user = \Collection::make($user)->toArray();

    //     if (empty($user) || empty($user['email']) || $user['email'] == '') {
    //         return redirect('member/login')->withErrors('取得 LINE 資料失敗，請直接於本站進行會員操作.')->send();
    //     }

    //     $member = \DB::table($this->config['table'])->where([['account', $user['email']], ['del', 0]])->first();

    //     if (empty($member)) {
    //         // 要寫入的資料
    //         $var = [
    //             'account'  => $user['email'],
    //             'password' => \Hash::make($user['email']),
    //             'name'     => $user['name'],
    //             'block'    => 0,
    //         ];

    //         $id = \DB::table($this->config['table'])->insertGetId($var);

    //         if (empty($id)) {
    //             return back()->withErrors(lang('member.register.check.add-fail'))->send();
    //         }

    // 		\Session::put('m_check', \Crypt::encrypt($id));
    //         \Session::put('mid', $id);

    //         return redirect("{$this->config['link_tag']}/data")->with('message', '加入會員成功, 預設密碼與您的信箱相同,請更改您的密碼.')->send();
    //     } else {
    //     	\Session::put('m_check', \Crypt::encrypt($member['id']));
    //         \Session::put('mid', $member['id']);

    //         $url = \Session::has('return_path') ? \Session::get('return_path') : "{$this->config['link_tag']}/index";

    //         return redirect($url)->with('message', lang('member.register.check.login'))->send();
    //     }
    // }
}