<?php 

namespace App\Controller\Web;

class NewsController 
{
    /**
     * $config.
     *
     * @var array
     */	
	protected $config;

	public function __construct()
	{
		$this->config = config('news');
	}	

	/**
	 * 首頁
	 */
	public function index( $level_1_class_id = null )
	{	
		// 分類
		$class = \DB::table($this->config['class_table'])->where([['act', 1], ['del', 0]])->orderBy('no', 'desc')->orderBy('id', 'asc')->get();
		$news_class = $class->keyBy('id')->toArray();
		$news_class_tree = $class->groupBy('class_id')->toArray();

		if(empty($level_1_class_id)){			
			$level_1_class_id = $news_class_tree[0][0]['id'];
		}

		// 頁數
		$page = \Request::query('page' , 1);
		$page_show = 10;

		// Data
		$data = \DB::table($this->config['table'])
					->when(!empty($level_1_class_id) , function($query) use ($level_1_class_id){
						return $query->where([ ['class_id' , $level_1_class_id] ]);						
					})
					->where([ ['act', 1], ['del', 0] ])
				   	->orderBy('date', 'desc')
				   	->orderBy('id', 'desc')
				   	->paginate($page_show, ['*'], 'page', $page)
				   	->withPath(base_path(\Request::path()))
				   	->appends(\Request::query());		  
 
		$view = \View::make("web.{$this->config['folder']}.index", compact( 'level_1_class_id' , 'news_class' , 'news_class_tree' , 'data' ))->render();

		return \Response::create($view)->send();
	}

	/**
	 * 搜尋
	 */
	public function search( )
	{	
		$keyword = urlencode(\Request::query('keyword'));

		// 頁數
		$page = \Request::query('page' , 1);
		$page_show = 10;

		// Data
		$data = \DB::table($this->config['table'])
					->where(function($query) use( $keyword){
						return $query->where('name' , 'like' , '%'.$keyword.'%')
							->orWhere('info' , 'like' , '%'.$keyword.'%');
					})
					->where([ ['act', 1], ['del', 0] ])
				   	->orderBy('date', 'desc')
				   	->orderBy('id', 'desc')
				   	->paginate($page_show, ['*'], 'page', $page)
				   	->withPath(base_path(\Request::path()))
				   	->appends(\Request::query());		  
 
		$view = \View::make("web.{$this->config['folder']}.index", compact( 'data' ))->render();

		return \Response::create($view)->send();
	}	

	/**
	 * 資料內頁
	 */
	public function detail( $level_1_class_id , $id)
	{
		// 分類
		$class = \DB::table($this->config['class_table'])->where([['act', 1], ['del', 0]])->orderBy('no', 'desc')->orderBy('id', 'asc')->get();
		$news_class = $class->keyBy('id');
		$news_class_tree = $class->groupBy('class_id');

		// Data
	    $data = \DB::table($this->config['table'])
	    		   ->where([['id', $id], ['act', 1], ['del', 0]])
	    		   ->first();

	    	   

	    $view = \View::make("web.{$this->config['folder']}.detail", compact( 'class_id' , 'news_class' , 'news_class_tree' , 'data' , 'prev' , 'next' ))->render();

	    return \Response::create($view)->send();	
	}
}