<?php

namespace App\Exports;

use Illuminate\Contracts\Support\Responsable;

use Maatwebsite\Excel\Concerns\FromQuery;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithMapping;
use Maatwebsite\Excel\Concerns\WithHeadings;

use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Events\BeforeExport;
// use Maatwebsite\Excel\Events\BeforeWriting;
// use Maatwebsite\Excel\Events\BeforeSheet;
use Maatwebsite\Excel\Events\AfterSheet;

/**
 * 導出
 *
 * @example https://docs.laravel-excel.com/3.1/exports/ 文檔
 * @example https://phpspreadsheet.readthedocs.io/en/latest/ PhpSpreadsheet 文檔
 */
class MemberExport implements FromQuery, WithEvents, WithHeadings, WithMapping, Responsable
{
	use Exportable;

    /**
     * $config.
     *
     * @var array
     */
	protected $config;

    /**
     * $titles.
     *
     * @var collection
     */
	protected $titles;

    /**
     * $fileName
     *
     * 導出的檔案名稱
     *
     * @var string
     */
	private $fileName = 'Member.xls';

	public function __construct()
	{
		$this->config = config('member');
		// 導出的檔案名稱
		$this->fileName = 'Member-' . \Date::now()->toDateTimeString() . '.xls';

		$this->titles = \Collection::make([
			['name' => '序號', 'width' => 5],
			['name' => '帳號/主信箱', 'width' => 30],
			['name' => '姓名', 'width' => 20],
			['name' => '性別', 'width' => 10],
			['name' => '生日', 'width' => 20],
			['name' => '聯絡電話', 'width' => 20],
			['name' => '手機號碼', 'width' => 20],
			['name' => '縣市', 'width' => 10],
			['name' => '鄉鎮', 'width' => 10],
			['name' => '郵遞區號', 'width' => 10],
			['name' => '詳細地址', 'width' => 50],
			['name' => '登入阻擋', 'width' => 10],
			['name' => '加入日期', 'width' => 20],
		]);
	}

    /**
     * 查詢資料
     */
    public function query()
    {
		$where = [['del', 0]];

        return \DB::table($this->config['table'])
				  ->where($where)
				  ->orderBy('created_at', 'desc');
    }

    /**
     * 格式化 - 列
     */
    public function map($row): array
    {
        return [
			'No'         => '=ROW()-2',
			'account'    => $row['account'],
			'name'       => $row['name'],
			'sex'        => $this->config['sex'][$row['sex']],
			'birthday'   => $row['birthday'],
			'tel'        => $row['tel'],
			'cel'        => $row['cel'],
			'country'    => $row['country'],
			'town'       => $row['town'],
			'zip'        => $row['zip'],
			'address'    => $row['address'],
			'block'      => $row['block'] == 1 ? '●' : '',
			'created_at' => $row['created_at'],
        ];
    }

    /**
     * 標題 標頭
     */
    public function headings(): array
    {
    	$title = config('system.web_title') . ' 會員資料';

        return [
			[$title],
			$this->titles->pluck('name')->toArray(),
        ];
    }

    public function registerEvents(): array
    {
        return [
            // BeforeExport::class => function(BeforeExport $event) {
            // 	$event->writer->getDelegate()->getProperties()
											 // ->setCreator('A')
											 // ->setLastModifiedBy('B')
											 // ->setTitle('C')
											 // ->setSubject('D')
											 // ->setDescription('E')
											 // ->setKeywords('F')
											 // ->setCategory('G');
            // },
            AfterSheet::class => function(AfterSheet $event) {

            	foreach ($this->titles->toArray() as $key => $value) {
            		$event->sheet->getDelegate()->getColumnDimensionByColumn($key + 1)->setWidth($value['width']);
            	}

            	// 整體預設
				$styleArray = [
				    'alignment' => [
				        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_LEFT,
				    ],
				];
            	$event->sheet->getDelegate()->getParent()->getDefaultStyle()->applyFromArray($styleArray);

            	// 第一列
            	$event->sheet->getDelegate()->mergeCells('A1:N1');
				$styleArray = [
				    'font' => [
				        'bold' => true,
				        'size' => 20,
				    ],
				    'alignment' => [
				        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
				    ],
				];
				$event->sheet->getDelegate()->getStyle('A1')->applyFromArray($styleArray);

				// 第二列
				$event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
				$styleArray = [
				    'alignment' => [
				        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
				    ],
				];
				$event->sheet->getDelegate()->getStyle('A2:T2')->applyFromArray($styleArray);

            },
        ];
    }
}